#ifndef createFields_H
#define createFields_H

    Info<< "Reading transportProperties\n" << endl;

    IOdictionary transportProperties
    (
        IOobject
        (
            "transportProperties",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );


//solvent viscosity
	dimensionedScalar nuS
    (
        transportProperties.lookup("nuS")
    );

	dimensionedScalar nuP
    (
        transportProperties.lookup("nuP")
    );
	dimensionedScalar rho
    (
        transportProperties.lookup("rho")
    );


//polymeric viscosity
	dimensionedScalar nu
	(
		nuS 
	);

    Info<< "Reading field p\n" << endl;
    volScalarField p
    (
        IOobject
        (
            "p",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );


    Info<< "Reading field U\n" << endl;
    volVectorField U
    (
        IOobject
        (
            "U",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

#   include "createPhi.H"

    label pRefCell = 0;
    scalar pRefValue = 0.0;
    setRefCell(p, mesh.solutionDict().subDict("PISO"), pRefCell, pRefValue);

//VelocityGradient Tensor
	volTensorField L
    (
        IOobject
        (
            "L",
            runTime.timeName(),
            mesh,
	    IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
	fvc::grad(U) //normal convention L=grad(U)^T
    );

//Rate of Deformation Tensor
	volTensorField D
    (
        IOobject
        (
            "D",
            runTime.timeName(),
            mesh,
	    IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        1.0/2.0*( L + L.T())
    );

//vorticity tensor
	volTensorField W
    (
        IOobject
        (
            "W",
            runTime.timeName(),
            mesh,
	    IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
	1.0/2.0*( L.T() - L) //ACHTUNG: DEFINITION OF L andersrum
    );

//RheologyControl Settings	

  Info<< "Reading RheologyControl\n" << endl;
    IOdictionary RheologyControl
    (
        IOobject
        (
            "RheologyControl",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

	scalar InnerIterations(
		readScalar(RheologyControl.lookup("InnerIterations"))
	);
	Info<< "Inner Iterations" << InnerIterations << nl << endl;

	Switch NonNewtonMatrix(
		 RheologyControl.lookup("NonNewtonMatrix")
	);	

	Switch FiberSuspension(
		 RheologyControl.lookup("FiberSuspension")
	);
		

tensor ut (1,0,0,0,1,0,0,0,1);
tensor zerotensor (0,0,0,0,0,0,0,0,0);
vector zerovector (0,0,0);
	
	volTensorField 	UnitTensor 
    	(
        IOobject
        (
            "UnitTensor",
            runTime.timeName(),
            mesh,
	    IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        	mesh,
		ut
    	);


#endif
