 /*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    nonNewton

Description
     incompressible, laminar flow of Fiber suspension (closed model from Advani/Folgar and Tucker).

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"

#include "argList.H"
#include "List.H"
# include "wallDist.H"

#ifndef namespaceFoam
#define namespaceFoam
    using namespace Foam;
#endif

int main(int argc, char *argv[])
{
#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"

#   include "createFields.H"
#   include "initContinuityErrs.H"


//if(NonNewtonMatrix){
# include "createFieldsMatrix.H"
//}

//if(MatrixModel=="PTT"){
#include "InitStressPTT.H"
//}


#	include "createFieldsFiber.H"
#	include "FiberInteractionModel.H"
scalar lambda = (r*r-1.0)/(r*r+1.0);


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#	include "FiberClosureHybrid.H"
#	include "calculateStressFiber.H"

//Foam::wallDist::wallDist(mesh,false);

 
    Info<< "\nStarting time loop\n" << endl;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;

	for ( int innerit = 1; innerit<= InnerIterations; innerit++){
#       include "readPISOControls.H"
#       include "CourantNo.H"

	L = fvc::grad(U);
	W = 1.0/2.0*( L.T() - L);
	D = 1.0/2.0*( L + L.T());
//	StressSolvent = nuS*( L + L.T() );
//	StressFiber = nuP*( L + L.T() );

if(FiberSuspension){
if(FiberClosure=="hybrid"){
#	include "FiberClosureHybrid.H"
#	include "calculateStressFiber.H"
}
if(FiberClosure=="IBOF"){
# 	include "FiberClosureIBOF.H"
#	include "calculateStressFiber.H"
}
}

if(NonNewtonMatrix){
//    Info<< "StressPTT" << endl;
if(MatrixModel=="PTT"){
	# include "StressPTT.H"
}
}

        fvVectorMatrix UEqn
        (
            fvm::ddt(U)
          + fvm::div(phi, U)
          - fvm::laplacian(nuS, U) //Newtonian Solvent Stress implicit
         );

if(NonNewtonMatrix){
	if(FiberSuspension){
		#include "solveUNonNewtonFiber.H"
   	}
	else{
		#include "solveUNonNewton.H"
	}
}
else if(FiberSuspension){
		#include "solveUNewtonFiber.H"
}
else{
		#include "solveUNewton.H"
}

         // --- PISO loop

        for (int corr=0; corr<nCorr; corr++)
        {
            volScalarField rUA = 1.0/UEqn.A();

            U = rUA*UEqn.H();
            phi = (fvc::interpolate(U) & mesh.Sf()) 
                + fvc::ddtPhiCorr(rUA, U, phi);

            adjustPhi(phi, U, p);

            for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
            {
                fvScalarMatrix pEqn
                (
                    fvm::laplacian(rUA, p) == fvc::div(phi)
                );

                pEqn.setReference(pRefCell, pRefValue);
                pEqn.solve();

                if (nonOrth == nNonOrthCorr)
                {
                    phi -= pEqn.flux();
                }
            }

#           include "continuityErrs.H"

            U -= rUA*fvc::grad(p);
            U.correctBoundaryConditions();
        }

if(FiberSuspension){	
if(FiberClosure=="hybrid"){
#	include "FiberClosureHybrid.H"	
#	include "calculateStressFiber.H"
}
if(FiberClosure=="IBOF"){
# 	include "FiberClosureIBOF.H"
#	include "calculateStressFiber.H"
}
	volScalarField Sp_imp1 = 6.0*CI*(pow(0.5*(D && D.T()),0.5));
	
	if(FiberVolfracWallFunc){
//Phan-Thien(2002)
		volScalarField CI_Field = 0.03*(1.0-Foam::exp(-0.224* FiberVolfracField*r));

		if ( FiberVolfrac*r > 1.3){
	//Bay
			CI_Field = 0.0184*Foam::exp(-0.7148* FiberVolfracField*r);
		}
		Sp_imp1 = 6.0*CI_Field*(pow(0.5*(D && D.T()),0.5));
	}
	
	
if(FiberClosure=="IBOF"){			
        fvTensorMatrix A2Eqn
	(
			fvm::ddt(A2) + fvm::div(phi, A2) + 
			fvm::Sp(Sp_imp1,A2) + fvm::Sp(ClosureA4_imp,A2)
	);
	solve(A2Eqn == SourceA2) ; 
}
else{
	fvTensorMatrix A2Eqn
			(
			fvm::ddt(A2) + fvm::div(phi, A2) );
	solve(A2Eqn == SourceA2) ; 

}
	
	//make symmetric...reason not found...numerical errors?
	A2.internalField() = symm(A2.internalField());
}
	
	} //end inner iterations loop

//	if(NonNewtonMatrix){
		# include "writeSolvent.H"
//	}
//	if(FiberSuspension){
		# include "writeFiber.H"
//	}

         runTime.write();

   //     Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
     //       << "  ClockTime = " << runTime.elapsedClockTime() << " s"
       //     << nl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
